library(dplyr)
library(gtools)
library(DT)
library(Lambda4)
library(psych)

shinyServer(function(input, output,session) {
  data <- reactive({
    inFile <- input$file1   
    if (is.null(inFile))
      return(NULL)
    dat <- read.csv(inFile$datapath)
    dat
    
  })
  
  vars <- reactive({
    dat <- data()
    vars <- names(dat)
    vars
    
  })
  
  
#Create Data Snippet 
  data2 <- reactive({
    inFile <- input$file1   
    if (is.null(inFile))
      return(NULL)
    dat <- read.csv(inFile$datapath)
    # Return first 5 rows
    dat
    head(dat, 5)
  })
  
  
    observeEvent(data(),{
    updateSelectInput(session, "columns","Select Items", choices = vars())
    updateSelectInput(session, "conv_val", "Select Convergent Validity Measure", choices = vars())
    updateSelectInput(session, "disc_val", "Select Discriminant Validity Measure", choices = vars())
    #col2-col15 generic names for items belonging to categories in content mapping 
    updateSelectInput(session, "col2", "Select Variables", choices = vars())
    updateSelectInput(session, "col3", "Select Variables", choices = vars())
    updateSelectInput(session, "col4", "Select Variables", choices = vars())
    updateSelectInput(session, "col5", "Select Variables", choices = vars())
    updateSelectInput(session, "col6", "Select Variables", choices = vars())
    updateSelectInput(session, "col7", "Select Variables", choices = vars())
    updateSelectInput(session, "col8", "Select Variables", choices = vars())
    updateSelectInput(session, "col9", "Select Variables", choices = vars())
    updateSelectInput(session, "col10", "Select Variables", choices = vars())
    updateSelectInput(session, "col11", "Select Variables", choices = vars())
    updateSelectInput(session, "col12", "Select Variables", choices = vars())
    updateSelectInput(session, "col13", "Select Variables", choices = vars())
    updateSelectInput(session, "col14", "Select Variables", choices = vars())
    updateSelectInput(session, "col15", "Select Variables", choices = vars())
    updateSelectInput(session, "reverse_scored", "Select Variables", choices = vars())
  })

#matrixvalues is final output table      
  matrixvalues <- reactive({
    
    #Code for progress bar----
    progress <- shiny::Progress$new()
    progress$set(message = "Computing Results", value = 0)
    # Close the progress when this reactive exits (even if there's an error)
    on.exit(progress$close())
    
    # Create a callback function to update progress bar.
    # Each time this is called:
    # - If `value` is NULL, it will move the progress bar 1/5 of the remaining
    #   distance. If non-NULL, it will set the progress to that value.
    # - It also accepts optional detail text.
    updateProgress <- function(value = NULL, detail = NULL) {
      
      if (is.null(value)) {
        value <- progress$getValue()
        value <- value + (progress$getMax() - value) / 5
      }
      progress$set(value = value, detail = detail)
    }

        
    inp_x <- length(input$columns) #number of variables in original scale
#Creates matrix of all possible subsets
    iterations <- permutations(2,inp_x,v = c(1,0), repeats.allowed = TRUE)
    iterations <- data.frame(perm = iterations)

    data_temp <- data()#gets data
    items <- subset(data_temp, select = c(input$columns))#subsets that data to only items

    #calculate total score for use in part-total correlation in the loop
    total.all <-  rowMeans(items, na.rm = TRUE)
    #assigns column names in iterations to match item names
    names(iterations) <- names(items)
    
    #calculates how many items are used in each set
    iterations$num_items <- as.integer(rowSums(iterations))
    
    #Create subsets for number of items in this range    
    iterations <- iterations[iterations$num_items>=input$Min & iterations$num_items<=input$Max,]

    #Used for end of loop and progress bar
    rows <- nrow(iterations)

    #pre-allocate vectors for the loop ----
    variables <- character(rows)
    alpha <- numeric(rows)
    lambda_2 <- numeric(rows)
    omegaH <- numeric(rows)
    r.part_whole <- numeric(rows)
    ConvValdty <- numeric(rows)
    DiscValdty <- numeric(rows)
    
    #Begin loop ----
    for (i in 1:rows){
      key <- as.logical(iterations[i,1:inp_x]) #identify items to select
      total <- rowMeans(items[key], na.rm = TRUE) #calculate subset total
      names1<- names(items[key]) #gather item names
      variables[i] <- paste(shQuote(names1, type="cmd"), collapse=", ")
      a_sub_temp <- suppressWarnings(psych::alpha(items[key]))
      alpha[i] <- round(a_sub_temp$total$raw_alpha, 3)
      #Do only if min items >2 and request Omega
      if (input$CalculateOmegaH=="yes" & input$Min >2) 
      {
        res_temp <- suppressWarnings(psych::omega(items[key], plot = FALSE))
        omegaH[i] <- round(res_temp$omega_, 3)
      }
      lambda_2[i] <- round(lambda2(items[key], missing = "complete", standardize = FALSE), 3)
      r.part_whole[i] <- round(cor(total,total.all), 3)
      if (!is.null(input$conv_val)) #Do only if there is a conv validity variable
      {
        ConvValdty[i] <- round(cor(total,  data()[,input$conv_val]), 3)
      }
      if (!is.null(input$disc_val)) #Do only if there is a Discr validity variable
      {
        DiscValdty[i] <- round(cor(total,  data()[,input$disc_val]), 3)
      }
            updateProgress((i/rows))
    }
    
    #Bind results that happen always
    iterations <- cbind(iterations, variables, alpha, lambda_2, r.part_whole)
    iterations$variables <- as.character(iterations$variables) #making sure this column is character so it can be filtered in output table
    
    #Bind only if Omega is calculated
    if (input$CalculateOmegaH == "yes" & input$Min > 2)
    {
      iterations <- cbind(iterations, omegaH)
    }
    
    #Bind conv_val results only if calculated
    if (!is.null(input$conv_val))
    {
      iterations <- cbind(iterations, ConvValdty)
    }    
    
    #Bind Disc_val results only if calculated
    if (!is.null(input$disc_val))
    {
      iterations <- cbind(iterations, DiscValdty)
    }

    iterations #return results table
  })
  observeEvent(input$values,{
    if(input$dim == "no" & input$polar == "no") {
      matrix_data <- matrixvalues()
    }
    else if(input$polar == "no" & input$dim == "yes" & input$count == 2) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col2))
      area2 <- subset(matrix_data, select = c(input$col3))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
    }
    else if(input$polar == "no" & input$dim == "yes" & input$count == 3) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col4))
      area2 <- subset(matrix_data, select = c(input$col5))
      area3 <- subset(matrix_data, select = c(input$col6))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
    }
    else if(input$polar == "no" & input$dim == "yes" & input$count == 4) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col7))
      area2 <- subset(matrix_data, select = c(input$col8))
      area3 <- subset(matrix_data, select = c(input$col9))
      area4 <- subset(matrix_data, select = c(input$col10))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
      matrix_data$num_items_CA4 <- as.integer(rowSums(area4))
    }
    else if(input$polar == "no" & input$dim == "yes" & input$count == 5) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col11))
      area2 <- subset(matrix_data, select = c(input$col12))
      area3 <- subset(matrix_data, select = c(input$col13))
      area4 <- subset(matrix_data, select = c(input$col14))
      area5 <- subset(matrix_data, select = c(input$col15))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
      matrix_data$num_items_CA4 <- as.integer(rowSums(area4))
      matrix_data$num_items_CA5 <- as.integer(rowSums(area5))
    }
    else if(input$polar == "yes" & input$dim == "yes" & input$count == 2) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col2))
      area2 <- subset(matrix_data, select = c(input$col3))
      area_new <- subset(matrix_data, select = c(input$reverse_scored))
      matrix_data$RS_items <- as.integer(rowSums(area_new))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
    }
    else if(input$polar == "yes" & input$dim == "yes" & input$count == 3) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col4))
      area2 <- subset(matrix_data, select = c(input$col5))
      area3 <- subset(matrix_data, select = c(input$col6))
      area_new <- subset(matrix_data, select = c(input$reverse_scored))
      matrix_data$RS_items <- as.integer(rowSums(area_new))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
    }
    else if(input$polar == "yes" & input$dim == "yes" & input$count == 4) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col7))
      area2 <- subset(matrix_data, select = c(input$col8))
      area3 <- subset(matrix_data, select = c(input$col9))
      area4 <- subset(matrix_data, select = c(input$col10))
      area_new <- subset(matrix_data, select = c(input$reverse_scored))
      matrix_data$RS_items <- as.integer(rowSums(area_new))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
      matrix_data$num_items_CA4 <- as.integer(rowSums(area4))
    }
    else if(input$polar == "yes" & input$dim == "yes" & input$count == 5) {
      matrix_data <- matrixvalues()
      area1 <- subset(matrix_data, select = c(input$col11))
      area2 <- subset(matrix_data, select = c(input$col12))
      area3 <- subset(matrix_data, select = c(input$col13))
      area4 <- subset(matrix_data, select = c(input$col14))
      area5 <- subset(matrix_data, select = c(input$col15))
      area_new <- subset(matrix_data, select = c(input$reverse_scored))
      matrix_data$RS_items <- as.integer(rowSums(area_new))
      matrix_data$num_items_CA1 <- as.integer(rowSums(area1))
      matrix_data$num_items_CA2 <- as.integer(rowSums(area2))
      matrix_data$num_items_CA3 <- as.integer(rowSums(area3))
      matrix_data$num_items_CA4 <- as.integer(rowSums(area4))
      matrix_data$num_items_CA5 <- as.integer(rowSums(area5))
    }
    else if(input$dim == "no" & input$polar == "yes") {
      matrix_data <- matrixvalues()  
      area_new <- subset(matrix_data, select = c(input$reverse_scored))
      matrix_data$RS_items <- rowSums(area_new)
    }
    
    drops <- c(input$columns)
      matrix_data <- matrix_data[ , !(names(matrix_data) %in% drops)]
      output$contents2 <- renderDataTable({    
        datatable(matrix_data, filter = 'top', rownames= FALSE,
                  
            
                  #Code for adding download buttons - work in progress
                  extensions = 'Buttons', options = list(
                  dom = 'Blfrtip',
                  buttons =
                    list(list(
                      extend = 'csv',
                      filename = 'oasisResults',
                      text = 'Download'
                    )
                    )
                    )
                  )

      }, server = FALSE) 
  }) 
  
   finalTable <- reactive({
        datas <- data2()
        datas
   })

  output$contents <- renderTable({    
    finalTable()
  })
  
})